@@ -1,4 +1,4 @@
-S.CARdissimilarity <- function(formula, family, data=NULL,  trials=NULL, W, Z, W.binary=TRUE, burnin, n.sample, thin=1, prior.mean.beta=NULL, prior.var.beta=NULL, prior.nu2=NULL, prior.tau2=NULL, verbose=TRUE)
+S.CARdissimilarity <- function(formula, family, data=NULL,  trials=NULL, W, Z, W.binary=TRUE, burnin, n.sample, thin=1, prior.mean.beta=NULL, prior.var.beta=NULL, prior.nu2=NULL, prior.tau2=NULL, MALA=TRUE, verbose=TRUE)
 {
 #### This is a wrapper function that calls one of
 ## binomial.properCAR
@@ -10,7 +10,7 @@ S.CARdissimilarity <- function(formula, family, data=NULL,  trials=NULL, W, Z, W
      if(family=="binomial")
      {
           if(is.null(trials)) stop("a binomial model was specified but the trials arugment was not specified", call.=FALSE)
-     model <- binomial.dissimilarityCAR(formula=formula, data=data,  trials=trials, W=W, Z=Z, W.binary=W.binary, burnin=burnin, n.sample=n.sample, thin=thin, prior.mean.beta=prior.mean.beta, prior.var.beta=prior.var.beta, prior.tau2=prior.tau2, verbose=verbose)
+     model <- binomial.dissimilarityCAR(formula=formula, data=data,  trials=trials, W=W, Z=Z, W.binary=W.binary, burnin=burnin, n.sample=n.sample, thin=thin, prior.mean.beta=prior.mean.beta, prior.var.beta=prior.var.beta, prior.tau2=prior.tau2, MALA=MALA, verbose=verbose)
      }else if(family=="gaussian")
      {
           if(!is.null(trials)) stop("you do not need a trials arugment as a binomial model was not specified", call.=FALSE)
@@ -18,7 +18,7 @@ S.CARdissimilarity <- function(formula, family, data=NULL,  trials=NULL, W, Z, W
      }else if(family=="poisson")
      {
           if(!is.null(trials)) stop("you do not need a trials arugment as a binomial model was not specified", call.=FALSE)
-     model <- poisson.dissimilarityCAR(formula=formula, data=data,  W=W, Z=Z, W.binary=W.binary, burnin=burnin, n.sample=n.sample, thin=thin, prior.mean.beta=prior.mean.beta, prior.var.beta=prior.var.beta, prior.tau2=prior.tau2, verbose=verbose)          
+     model <- poisson.dissimilarityCAR(formula=formula, data=data,  W=W, Z=Z, W.binary=W.binary, burnin=burnin, n.sample=n.sample, thin=thin, prior.mean.beta=prior.mean.beta, prior.var.beta=prior.var.beta, prior.tau2=prior.tau2, MALA=MALA, verbose=verbose)          
      }else
      {
      stop("the family arugment is not one of `binomial', `gaussian' or `poisson'.", call.=FALSE)     
