@@ -18,10 +18,10 @@ X.mean <- frame.results$X.mean
 X.indicator <- frame.results$X.indicator 
 offset <- frame.results$offset
 Y <- frame.results$Y
-Y.miss <- frame.results$Y.miss
 which.miss <- frame.results$which.miss
 n.miss <- frame.results$n.miss  
-    
+Y.DA <- Y
+
     
 #### Check on MALA argument
     if(length(MALA)!=1) stop("MALA is not length 1.", call.=FALSE)
@@ -35,8 +35,7 @@ int.check <- K-sum(ceiling(trials)==floor(trials))
     if(int.check > 0) stop("the numbers of trials has non-integer values.", call.=FALSE)
     if(min(trials)<=0) stop("the numbers of trials has zero or negative values.", call.=FALSE)
 failures <- trials - Y
-failures.miss <- failures
-failures.miss[which.miss==0] <- median(failures, na.rm=TRUE)
+failures.DA <- trials - Y.DA
     if(sum(Y>trials, na.rm=TRUE)>0) stop("the response variable has larger values that the numbers of trials.", call.=FALSE)
 
 
@@ -86,7 +85,8 @@ phi <- rnorm(n=K, mean=rep(0,K), sd=res.sd)
 tau2 <- var(phi) / 10
 theta <- rnorm(n=K, mean=rep(0,K), sd=res.sd)
 sigma2 <- var(theta) / 10
-
+lp <- as.numeric(X.standardised %*% beta) + phi + theta + offset
+prob <- exp(lp)  / (1 + exp(lp))
 
 
 ###############################    
@@ -98,7 +98,6 @@ samples.beta <- array(NA, c(n.keep, p))
 samples.re <- array(NA, c(n.keep, K))
 samples.tau2 <- array(NA, c(n.keep, 1))
 samples.sigma2 <- array(NA, c(n.keep, 1))
-samples.deviance <- array(NA, c(n.keep, 1))
 samples.like <- array(NA, c(n.keep, K))
 samples.fitted <- array(NA, c(n.keep, K))
 if(n.miss>0) samples.Y <- array(NA, c(n.keep, n.miss))
@@ -156,16 +155,28 @@ tau2.posterior.shape <- prior.tau2[1] + 0.5 * (K-n.islands)
 #### Create the MCMC samples      
     for(j in 1:n.sample)
     {
+    ####################################
+    ## Sample from Y - data augmentation
+    ####################################
+        if(n.miss>0)
+        {
+        Y.DA[which.miss==0] <- rbinom(n=n.miss, size=trials[which.miss==0], prob=prob[which.miss==0])
+        failures.DA <- trials - Y.DA
+        }else
+        {}
+        
+        
+        
     ####################
     ## Sample from beta
     ####################
     offset.temp <- phi + offset + theta
         if(p>2)
         {
-        temp <- binomialbetaupdateMALA(X.standardised, K, p, beta, offset.temp, Y.miss, failures.miss, trials, prior.mean.beta, prior.var.beta, which.miss, n.beta.block, proposal.sd.beta, list.block)
+        temp <- binomialbetaupdateMALA(X.standardised, K, p, beta, offset.temp, Y.DA, failures.DA, trials, prior.mean.beta, prior.var.beta, n.beta.block, proposal.sd.beta, list.block)
         }else
         {
-        temp <- binomialbetaupdateRW(X.standardised, K, p, beta, offset.temp, Y.miss, failures.miss, prior.mean.beta, prior.var.beta, which.miss, proposal.sd.beta)
+        temp <- binomialbetaupdateRW(X.standardised, K, p, beta, offset.temp, Y.DA, failures.DA, prior.mean.beta, prior.var.beta, proposal.sd.beta)
         }
     beta <- temp[[1]]
     accept[1] <- accept[1] + temp[[2]]
@@ -179,15 +190,15 @@ tau2.posterior.shape <- prior.tau2[1] + 0.5 * (K-n.islands)
     beta.offset <- X.standardised %*% beta + theta + offset
         if(MALA)
         {
-        temp1 <- binomialcarupdateMALA(Wtriplet=W.triplet, Wbegfin=W.begfin, Wtripletsum=W.triplet.sum, nsites=K, phi=phi, tau2=tau2, y=Y.miss, failures=failures.miss, trials=trials, phi_tune=proposal.sd.phi, rho=1, offset=beta.offset, which.miss)
+        temp1 <- binomialcarupdateMALA(Wtriplet=W.triplet, Wbegfin=W.begfin, Wtripletsum=W.triplet.sum, nsites=K, phi=phi, tau2=tau2, y=Y.DA, failures=failures.DA, trials=trials, phi_tune=proposal.sd.phi, rho=1, offset=beta.offset)
         }else
         {
-        temp1 <- binomialcarupdateRW(Wtriplet=W.triplet, Wbegfin=W.begfin, Wtripletsum=W.triplet.sum, nsites=K, phi=phi, tau2=tau2, y=Y.miss, failures=failures.miss, phi_tune=proposal.sd.phi, rho=1, offset=beta.offset, which.miss)
+        temp1 <- binomialcarupdateRW(Wtriplet=W.triplet, Wbegfin=W.begfin, Wtripletsum=W.triplet.sum, nsites=K, phi=phi, tau2=tau2, y=Y.DA, failures=failures.DA, phi_tune=proposal.sd.phi, rho=1, offset=beta.offset)
         }
     phi <- temp1[[1]]
     phi[which(islands==1)] <- phi[which(islands==1)] - mean(phi[which(islands==1)])
     accept[3] <- accept[3] + temp1[[2]]
-    accept[4] <- accept[4] + K - n.miss
+    accept[4] <- accept[4] + K
      
     
 
@@ -197,15 +208,15 @@ tau2.posterior.shape <- prior.tau2[1] + 0.5 * (K-n.islands)
     beta.offset <- as.numeric(X.standardised %*% beta) + phi + offset
         if(MALA)
         {
-        temp2 <- binomialindepupdateMALA(nsites=K, theta=theta, sigma2=sigma2, y=Y.miss, failures=failures.miss, trials=trials, theta_tune=proposal.sd.theta, offset=beta.offset, which.miss) 
+        temp2 <- binomialindepupdateMALA(nsites=K, theta=theta, sigma2=sigma2, y=Y.DA, failures=failures.DA, trials=trials, theta_tune=proposal.sd.theta, offset=beta.offset) 
         }else
         {
-        temp2 <- binomialindepupdateRW(nsites=K, theta=theta, sigma2=sigma2, y=Y.miss, failures=failures.miss, theta_tune=proposal.sd.theta, offset=beta.offset, which.miss) 
+        temp2 <- binomialindepupdateRW(nsites=K, theta=theta, sigma2=sigma2, y=Y.DA, failures=failures.DA, theta_tune=proposal.sd.theta, offset=beta.offset) 
         }
     theta <- temp2[[1]]
     theta <- theta - mean(theta)    
     accept[5] <- accept[5] + temp2[[2]]
-    accept[6] <- accept[6] + K - n.miss          
+    accept[6] <- accept[6] + K          
      
 
 
@@ -248,10 +259,9 @@ tau2.posterior.shape <- prior.tau2[1] + 0.5 * (K-n.islands)
         samples.re[ele, ] <- phi + theta
         samples.tau2[ele, ] <- tau2
         samples.sigma2[ele, ] <- sigma2
-        samples.deviance[ele, ] <- deviance
         samples.like[ele, ] <- like
         samples.fitted[ele, ] <- fitted
-            if(n.miss>0) samples.Y[ele, ] <- rbinom(n=n.miss, size=trials[which.miss==0], prob=prob[which.miss==0])
+            if(n.miss>0) samples.Y[ele, ] <- Y.DA[which.miss==0]
         }else
         {
         }
@@ -314,36 +324,17 @@ accept.final <- c(accept.beta, accept.phi, accept.theta, accept.tau2, accept.sig
 names(accept.final) <- c("beta", "phi", "theta", "tau2", "sigma2")
 
      
-#### Deviance information criterion (DIC)
-median.beta <- apply(samples.beta, 2, median)
-median.re <- apply(samples.re, 2, median)
-median.logit <- as.numeric(X.standardised %*% median.beta) + median.re + offset    
-median.prob <- exp(median.logit)  / (1 + exp(median.logit))
-fitted.median <- trials * median.prob
-deviance.fitted <- -2 * sum(dbinom(x=Y, size=trials, prob=median.prob, log=TRUE), na.rm=TRUE)
-p.d <- median(samples.deviance) - deviance.fitted
-DIC <- 2 * median(samples.deviance) - deviance.fitted     
+#### Compute the fitted deviance
+mean.beta <- apply(samples.beta, 2, mean)
+mean.re <- apply(samples.re, 2, mean)
+mean.logit <- as.numeric(X.standardised %*% mean.beta) + mean.re + offset    
+mean.prob <- exp(mean.logit)  / (1 + exp(mean.logit))
+fitted.mean <- trials * mean.prob
+deviance.fitted <- -2 * sum(dbinom(x=Y, size=trials, prob=mean.prob, log=TRUE), na.rm=TRUE)
 
 
-#### Watanabe-Akaike Information Criterion (WAIC)
-LPPD <- sum(log(apply(samples.like,2,mean)), na.rm=TRUE)
-p.w <- sum(apply(log(samples.like),2,var), na.rm=TRUE)
-WAIC <- -2 * (LPPD - p.w)
-
-
-#### Compute the Conditional Predictive Ordinate
-CPO <- rep(NA, K)
-     for(j in 1:K)
-     {
-     CPO[j] <- 1/median((1 / dbinom(x=Y[j], size=trials[j], prob=(samples.fitted[ ,j] / trials[j]))))    
-     }
-LMPL <- sum(log(CPO), na.rm=TRUE)       
-     
-     
-#### Compute the % deviance explained
-fit.null <- glm(cbind(Y, failures)~1, offset=offset, family="binomial")$fitted.values
-deviance.null <- -2 * sum(dbinom(x=Y[which(!is.na(Y))], size=trials[which(!is.na(Y))], prob=fit.null, log=TRUE), na.rm=TRUE)
-percent_dev_explained <- 100 * (deviance.null - deviance.fitted) / deviance.null
+#### Model fit criteria
+modelfit <- common.modelfit(samples.like, deviance.fitted)
 
 
 #### transform the parameters back to the origianl covariate scale.
@@ -370,17 +361,13 @@ summary.results[ , 4:7] <- round(summary.results[ , 4:7], 1)
 
 
 #### Create the fitted values and residuals
-fitted.values <- apply(samples.fitted, 2, median)
+fitted.values <- apply(samples.fitted, 2, mean)
 response.residuals <- as.numeric(Y) - fitted.values
-pearson.residuals <- response.residuals /sqrt(fitted.values * (1 - median.prob))
-deviance.residuals <- sign(response.residuals) * sqrt(2 * (Y * log(Y/fitted.values) + (trials-Y) * log((trials-Y)/(trials - fitted.values))))
-residuals <- data.frame(response=response.residuals, pearson=pearson.residuals, deviance=deviance.residuals)
+pearson.residuals <- response.residuals /sqrt(fitted.values * (1 - mean.prob))
+residuals <- data.frame(response=response.residuals, pearson=pearson.residuals)
 
 
 #### Compile and return the results
-loglike <- (-0.5 * deviance.fitted)
-modelfit <- c(DIC, p.d, WAIC, p.w, LMPL, loglike, percent_dev_explained)
-names(modelfit) <- c("DIC", "p.d", "WAIC", "p.w", "LMPL", "loglikelihood", "Percentage deviance explained")
 model.string <- c("Likelihood model - Binomial (logit link function)", "\nRandom effects model - BYM CAR\n")
 if(n.miss==0) samples.Y = NA
 
