@@ -18,7 +18,6 @@ X.mean <- frame.results$X.mean
 X.indicator <- frame.results$X.indicator 
 offset <- frame.results$offset
 Y <- frame.results$Y
-which.miss <- as.numeric(!is.na(Y))
 regression.vec <- frame.results$regression.vec
 beta <- frame.results$beta
 
@@ -127,7 +126,6 @@ samples.tau2 <- array(NA, c(n.keep, 1))
 samples.Z <- array(NA, c(n.keep, K))
 samples.lambda <- array(NA, c(n.keep, G))
 samples.delta <- array(NA, c(n.keep, 1))
-samples.deviance <- array(NA, c(n.keep, 1))
 samples.like <- array(NA, c(n.keep, K))
 samples.fitted <- array(NA, c(n.keep, K))
 
@@ -191,10 +189,10 @@ W.begfin <- W.quants$W.begfin
         offset.temp <- phi + offset + lambda[Z]
             if(p>2)
             {
-            temp <- binomialbetaupdateMALA(X.standardised, K, p, beta, offset.temp, Y, failures, trials, prior.mean.beta, prior.var.beta, which.miss, n.beta.block, proposal.sd.beta, list.block)
+            temp <- binomialbetaupdateMALA(X.standardised, K, p, beta, offset.temp, Y, failures, trials, prior.mean.beta, prior.var.beta, n.beta.block, proposal.sd.beta, list.block)
             }else
             {
-            temp <- binomialbetaupdateRW(X.standardised, K, p, beta, offset.temp, Y, failures, prior.mean.beta, prior.var.beta, which.miss, proposal.sd.beta)
+            temp <- binomialbetaupdateRW(X.standardised, K, p, beta, offset.temp, Y, failures, prior.mean.beta, prior.var.beta, proposal.sd.beta)
             }
         beta <- temp[[1]]
         accept[7] <- accept[7] + temp[[2]]
@@ -211,10 +209,10 @@ W.begfin <- W.quants$W.begfin
      beta.offset <- regression.vec + offset  + lambda[Z]
         if(MALA)
         {
-        temp1 <- binomialcarupdateMALA(Wtriplet=W.triplet, Wbegfin=W.begfin, Wtripletsum=W.triplet.sum, nsites=K, phi=phi, tau2=tau2, y=Y, failures=failures, trials=trials, phi_tune=proposal.sd.phi, rho=1, offset=beta.offset, which.miss)
+        temp1 <- binomialcarupdateMALA(Wtriplet=W.triplet, Wbegfin=W.begfin, Wtripletsum=W.triplet.sum, nsites=K, phi=phi, tau2=tau2, y=Y, failures=failures, trials=trials, phi_tune=proposal.sd.phi, rho=1, offset=beta.offset)
         }else
         {
-        temp1 <- binomialcarupdateRW(Wtriplet=W.triplet, Wbegfin=W.begfin, Wtripletsum=W.triplet.sum, nsites=K, phi=phi, tau2=tau2, y=Y, failures=failures, phi_tune=proposal.sd.phi, rho=1, offset=beta.offset, which.miss)
+        temp1 <- binomialcarupdateRW(Wtriplet=W.triplet, Wbegfin=W.begfin, Wtripletsum=W.triplet.sum, nsites=K, phi=phi, tau2=tau2, y=Y, failures=failures, phi_tune=proposal.sd.phi, rho=1, offset=beta.offset)
         }
      phi <- temp1[[1]]
           for(i in 1:G)
@@ -223,7 +221,7 @@ W.begfin <- W.quants$W.begfin
           }
      accept[1] <- accept[1] + temp1[[2]]
      accept[2] <- accept[2] + K    
-  
+
          
      
      ##################
@@ -238,7 +236,7 @@ W.begfin <- W.quants$W.begfin
     #####################
     ## Sample from lambda
     #####################
-    proposal <- c(-1000, lambda, 1000)
+     proposal <- c(-1000, lambda, 1000)
           for(i in 1:G)
           {
            proposal[(i+1)] <- rtruncnorm(n=1, a=proposal[i], b=proposal[(i+2)], mean=proposal[(i+1)], sd=proposal.sd.lambda)    
@@ -317,7 +315,6 @@ W.begfin <- W.quants$W.begfin
         samples.tau2[ele, ] <- tau2
         samples.Z[ele, ] <- Z
         samples.delta[ele, ] <- delta
-        samples.deviance[ele, ] <- deviance
         samples.like[ele, ] <- like
         samples.fitted[ele, ] <- fitted
         if(!is.null(X)) samples.beta[ele, ] <- beta    
@@ -400,47 +397,28 @@ accept.tau2 <- 100
     }
 
 
-#### Deviance information criterion (DIC)
-median.phi <- apply(samples.phi, 2, median)
-median.Z <- round(apply(samples.Z,2,median),0)
-median.lambda <- apply(samples.lambda,2,median)
+#### Compute the fitted deviance
+mean.phi <- apply(samples.phi, 2, mean)
+mean.Z <- round(apply(samples.Z,2,mean),0)
+mean.lambda <- apply(samples.lambda,2,mean)
     if(!is.null(X))
     {
-    median.beta <- apply(samples.beta,2,median)
-    regression.vec <- as.numeric(X.standardised %*% median.beta)   
+    mean.beta <- apply(samples.beta,2,mean)
+    regression.vec <- as.numeric(X.standardised %*% mean.beta)   
     }else
     {
     regression.vec <- rep(0,K)
     }
-median.logit <- median.lambda[median.Z] + median.phi + regression.vec + offset
-median.prob <- exp(median.logit)  / (1 + exp(median.logit))
-fitted.deviance <- dbinom(x=Y, size=trials, prob=median.prob)
-deviance.fitted <- -2 * sum(dbinom(x=Y, size=trials, prob=median.prob, log=TRUE))
-p.d <- median(samples.deviance) - deviance.fitted
-DIC <- 2 * median(samples.deviance) - deviance.fitted     
+mean.logit <- mean.lambda[mean.Z] + mean.phi + regression.vec + offset
+mean.prob <- exp(mean.logit)  / (1 + exp(mean.logit))
+fitted.deviance <- dbinom(x=Y, size=trials, prob=mean.prob)
+deviance.fitted <- -2 * sum(dbinom(x=Y, size=trials, prob=mean.prob, log=TRUE))
 
 
-#### Watanabe-Akaike Information Criterion (WAIC)
-LPPD <- sum(log(apply(samples.like,2,mean)), na.rm=TRUE)
-p.w <- sum(apply(log(samples.like),2,var), na.rm=TRUE)
-WAIC <- -2 * (LPPD - p.w)
-
-
-#### Compute the Conditional Predictive Ordinate
-CPO <- rep(NA, K)
-     for(j in 1:K)
-     {
-     CPO[j] <- 1/median((1 / dbinom(x=Y[j], size=trials[j], prob=(samples.fitted[ ,j] / trials[j]))))    
-     }
-LMPL <- sum(log(CPO))  
+#### Model fit criteria
+modelfit <- common.modelfit(samples.like, deviance.fitted)
 
      
-#### Compute the % deviance explained
-fit.null <- glm(cbind(Y, failures)~1, offset=offset, family="binomial")$fitted.values
-deviance.null <- -2 * sum(dbinom(x=Y[which(!is.na(Y))], size=trials[which(!is.na(Y))], prob=fit.null, log=TRUE), na.rm=TRUE)
-percent_dev_explained <- 100 * (deviance.null - deviance.fitted) / deviance.null
-
-
 #### transform the parameters back to the origianl covariate scale.
     if(!is.null(X))
     {    
@@ -485,23 +463,19 @@ summary.lambda <- summary.lambda[Z.used, ]
 
 
 #### Create the fitted values and residuals
-fitted.values <- apply(samples.fitted, 2, median)
+fitted.values <- apply(samples.fitted, 2, mean)
 response.residuals <- as.numeric(Y) - fitted.values
-pearson.residuals <- response.residuals /sqrt(fitted.values * (1 - median.prob))
-deviance.residuals <- sign(response.residuals) * sqrt(2 * (Y * log(Y/fitted.values) + (trials-Y) * log((trials-Y)/(trials - fitted.values))))
-residuals <- data.frame(response=response.residuals, pearson=pearson.residuals, deviance=deviance.residuals)
+pearson.residuals <- response.residuals /sqrt(fitted.values * (1 - mean.prob))
+residuals <- data.frame(response=response.residuals, pearson=pearson.residuals)
 
 
 
 #### Compile and return the results
-loglike <- (-0.5 * deviance.fitted)
-modelfit <- c(DIC, p.d, WAIC, p.w, LMPL, loglike, percent_dev_explained)
-names(modelfit) <- c("DIC", "p.d", "WAIC", "p.w", "LMPL", "loglikelihood", "Percentage deviance explained")
 model.string <- c("Likelihood model - Binomial (logit link function)", "\nRandom effects  model - Localised CAR model\n")
 if(is.null(X)) samples.beta.orig = NA
 
 samples <- list(beta=mcmc(samples.beta.orig), phi=mcmc(samples.phi), lambda=mcmc(samples.lambda), Z=mcmc(samples.Z), tau2=mcmc(samples.tau2), delta=mcmc(samples.delta), fitted=mcmc(samples.fitted))          
-results <- list(summary.results=summary.results, samples=samples, fitted.values=fitted.values, residuals=residuals, modelfit=modelfit, accept=accept.final, localised.structure=median.Z,  formula=formula, model=model.string, X=X)
+results <- list(summary.results=summary.results, samples=samples, fitted.values=fitted.values, residuals=residuals, modelfit=modelfit, accept=accept.final, localised.structure=mean.Z,  formula=formula, model=model.string, X=X)
 class(results) <- "CARBayes"
 
 #### Finish by stating the time taken    
