@@ -18,12 +18,9 @@ X.mean <- frame.results$X.mean
 X.indicator <- frame.results$X.indicator 
 offset <- frame.results$offset
 Y <- frame.results$Y
-Y.miss <- frame.results$Y.miss
 which.miss <- frame.results$which.miss
 n.miss <- frame.results$n.miss  
-Y.short <- Y[which.miss==1]
-X.short <- X.standardised[which.miss==1, ]
-offset.short <- offset[which.miss==1]
+Y.DA <- Y
 
 
 #### Dissimilarity metric matrix
@@ -83,6 +80,7 @@ phi <- rnorm(n=K, mean=rep(0,K), sd=res.sd)
 tau2 <- var(phi) / 10
 nu2 <- tau2
 alpha <- runif(n=q, min=rep(0,q), max=rep(alpha.max/(2+q))) 
+fitted <- as.numeric(X.standardised %*% beta) + phi + offset
 
 
 
@@ -96,7 +94,6 @@ samples.phi <- array(NA, c(n.keep, K))
 samples.nu2 <- array(NA, c(n.keep, 1))
 samples.tau2 <- array(NA, c(n.keep, 1))
 samples.alpha <- array(NA, c(n.keep, q))
-samples.deviance <- array(NA, c(n.keep, 1))
 samples.like <- array(NA, c(n.keep, K))
 samples.fitted <- array(NA, c(n.keep, K))
     if(n.miss>0) samples.Y <- array(NA, c(n.keep, n.miss))
@@ -107,7 +104,7 @@ accept <- c(0,0)
 accept.all <- c(0,0)
 proposal.sd.alpha <- 0.02 * alpha.max
 tau2.posterior.shape <- prior.tau2[1] + 0.5 * K
-nu2.posterior.shape <- prior.nu2[1] + 0.5*(K-n.miss)  
+nu2.posterior.shape <- prior.nu2[1] + 0.5*K  
 
 
 
@@ -158,7 +155,7 @@ det.Q <- sum(log(diag(chol.spam(Q))))
 
 
 #### Beta update quantities
-data.precision.beta <- t(X.short) %*% X.short
+data.precision.beta <- t(X.standardised) %*% X.standardised
 	if(length(prior.var.beta)==1)
 	{
 	prior.precision.beta <- 1 / prior.var.beta
@@ -187,13 +184,24 @@ data.precision.beta <- t(X.short) %*% X.short
 #### Create the MCMC samples 
     for(j in 1:n.sample)
     {
-	####################
+    ####################################
+    ## Sample from Y - data augmentation
+    ####################################
+        if(n.miss>0)
+        {
+        Y.DA[which.miss==0] <- rnorm(n=n.miss, mean=fitted[which.miss==0], sd=sqrt(nu2))    
+        }else
+        {}
+        
+        
+        
+    ####################
 	## Sample from beta
 	####################
 	fc.precision <- prior.precision.beta + data.precision.beta / nu2
 	fc.var <- solve(fc.precision)
-	beta.offset <- as.numeric(Y.short - offset.short - phi[which.miss==1])
-	beta.offset2 <- t(X.short) %*% beta.offset / nu2 + prior.precision.beta %*% prior.mean.beta
+	beta.offset <- as.numeric(Y.DA - offset - phi)
+	beta.offset2 <- t(X.standardised) %*% beta.offset / nu2 + prior.precision.beta %*% prior.mean.beta
 	fc.mean <- fc.var %*% beta.offset2
 	chol.var <- t(chol(fc.var))
 	beta <- fc.mean + chol.var %*% rnorm(p)  
@@ -203,8 +211,8 @@ data.precision.beta <- t(X.short) %*% X.short
 	##################
 	## Sample from nu2
 	##################
-    fitted.current <-  as.numeric(X.short %*% beta) + phi[which.miss==1] + offset.short
-    nu2.posterior.scale <- prior.nu2[2] + 0.5 * sum((Y.short - fitted.current)^2)
+    fitted.current <-  as.numeric(X.standardised %*% beta) + phi + offset
+    nu2.posterior.scale <- prior.nu2[2] + 0.5 * sum((Y.DA - fitted.current)^2)
     nu2 <- 1 / rgamma(1, nu2.posterior.shape, scale=(1/nu2.posterior.scale)) 
 
     
@@ -212,8 +220,8 @@ data.precision.beta <- t(X.short) %*% X.short
 	####################
 	## Sample from phi
 	####################
-    offset.phi <- (Y.miss - as.numeric(X.standardised %*% beta) - offset) / nu2    
-    phi <- gaussiancarupdate(Wtriplet=W.triplet, Wbegfin=W.begfin, W.triplet.sum, nsites=K, phi=phi, tau2=tau2, rho=rho, nu2=nu2, offset=offset.phi, which.miss)
+    offset.phi <- (Y.DA - as.numeric(X.standardised %*% beta) - offset) / nu2    
+    phi <- gaussiancarupdate(Wtriplet=W.triplet, Wbegfin=W.begfin, W.triplet.sum, nsites=K, phi=phi, tau2=tau2, rho=rho, nu2=nu2, offset=offset.phi)
     phi <- phi - mean(phi)
 		
 
@@ -292,10 +300,9 @@ data.precision.beta <- t(X.short) %*% X.short
         samples.nu2[ele, ] <- nu2
         samples.tau2[ele, ] <- tau2
         samples.alpha[ele, ] <- alpha
-        samples.deviance[ele, ] <- deviance
         samples.like[ele, ] <- like
         samples.fitted[ele, ] <- fitted
-            if(n.miss>0) samples.Y[ele, ] <- rnorm(n=n.miss, mean=fitted[which.miss==0], sd=sqrt(nu2))
+            if(n.miss>0) samples.Y[ele, ] <- Y.DA[which.miss==0]
         }else
         {}
 
@@ -345,35 +352,16 @@ accept.final <- c(100, 100, 100, 100, accept.alpha)
 names(accept.final) <- c("beta", "phi", "nu2", "tau2", "alpha")
  
         
-#### Deviance information criterion (DIC)
-median.beta <- apply(samples.beta, 2, median)
-median.phi <- apply(samples.phi, 2, median)
-fitted.median <- X.standardised %*% median.beta + median.phi + offset
-nu2.median <- median(samples.nu2)
-deviance.fitted <- -2 * sum(dnorm(Y, mean = fitted.median, sd = rep(sqrt(nu2.median),K), log = TRUE), na.rm=TRUE)
-p.d <- median(samples.deviance) - deviance.fitted
-DIC <- 2 * median(samples.deviance) - deviance.fitted    
+#### Compute the fitted deviance
+mean.beta <- apply(samples.beta, 2, mean)
+mean.phi <- apply(samples.phi, 2, mean)
+fitted.mean <- X.standardised %*% mean.beta + mean.phi + offset
+nu2.mean <- mean(samples.nu2)
+deviance.fitted <- -2 * sum(dnorm(Y, mean = fitted.mean, sd = rep(sqrt(nu2.mean),K), log = TRUE), na.rm=TRUE)
+
    
-   
-#### Watanabe-Akaike Information Criterion (WAIC)
-LPPD <- sum(log(apply(samples.like,2,mean)), na.rm=TRUE)
-p.w <- sum(apply(log(samples.like),2,var), na.rm=TRUE)
-WAIC <- -2 * (LPPD - p.w)
-
-
-#### Compute the Conditional Predictive Ordinate
-CPO <- rep(NA, K)
-     for(j in 1:K)
-     {
-     CPO[j] <- 1/median((1 / dnorm(Y[j], mean=samples.fitted[ ,j], sd=sqrt(samples.nu2))))    
-     }
-LMPL <- sum(log(CPO), na.rm=TRUE)   
-     
-
-#### Compute the % deviance explained
-deviance.null <- sum((Y - mean(Y, na.rm=T))^2, na.rm=T)
-deviance.fit <- sum((Y - fitted.median)^2, na.rm=T)
-percent_dev_explained <- 100 * (deviance.null - deviance.fit) / deviance.null
+#### Model fit criteria
+modelfit <- common.modelfit(samples.like, deviance.fitted)
 
 
 #### transform the parameters back to the origianl covariate scale.
@@ -424,11 +412,10 @@ summary.results[ , 4:7] <- round(summary.results[ , 4:7], 1)
 
 
 #### Create the Fitted values and residuals
-fitted.values <- apply(samples.fitted, 2, median)
+fitted.values <- apply(samples.fitted, 2, mean)
 response.residuals <- as.numeric(Y) - fitted.values
-pearson.residuals <- response.residuals /sqrt(nu2.median)
-deviance.residuals <- sign(response.residuals) * sqrt((Y-fitted.values)^2/nu2.median)
-residuals <- data.frame(response=response.residuals, pearson=pearson.residuals, deviance=deviance.residuals)
+pearson.residuals <- response.residuals /sqrt(nu2.mean)
+residuals <- data.frame(response=response.residuals, pearson=pearson.residuals)
 
  
 #### Create the posterior medians for the neighbourhood matrix W
@@ -472,9 +459,6 @@ W.posterior <- array(NA, c(K,K))
 
 
 ## Compile and return the results
-loglike <- (-0.5 * deviance.fitted)
-modelfit <- c(DIC, p.d, WAIC, p.w, LMPL, loglike, percent_dev_explained)
-names(modelfit) <- c("DIC", "p.d", "WAIC", "p.w", "LMPL", "loglikelihood", "Percentage deviance explained")
     if(W.binary)
     {
     model.string <- c("Likelihood model - Gaussian (identity link function)", "\nRandom effects model - Binary dissimilarity CAR", "\nDissimilarity metrics - ", rownames(summary.alpha), "\n")     
