@@ -1,4 +1,4 @@
-poisson.MVlerouxCAR <- function(formula, data=NULL,  W, burnin, n.sample, thin=1, prior.mean.beta=NULL, prior.var.beta=NULL, prior.Sigma.df=NULL, prior.Sigma.scale=NULL, fix.rho=FALSE, rho=NULL, MALA=TRUE, verbose=TRUE)
+poisson.MVlerouxCAR <- function(formula, data=NULL,  W, burnin, n.sample, thin=1, prior.mean.beta=NULL, prior.var.beta=NULL, prior.Sigma.df=NULL, prior.Sigma.scale=NULL, rho=NULL, MALA=TRUE, verbose=TRUE)
 {
 ##############################################
 #### Format the arguments and check for errors
@@ -9,7 +9,7 @@ a <- common.verbose(verbose)
     
 #### Frame object
 frame.results <- common.frame(formula, data, "poisson")
-N.all <- frame.results$n
+K <- frame.results$n
 p <- frame.results$p
 X <- frame.results$X
 X.standardised <- frame.results$X.standardised
@@ -18,11 +18,25 @@ X.mean <- frame.results$X.mean
 X.indicator <- frame.results$X.indicator 
 offset <- frame.results$offset
 Y <- frame.results$Y
-Y.miss <- frame.results$Y.miss
 which.miss <- frame.results$which.miss
 n.miss <- frame.results$n.miss  
+J <- ncol(Y)
+N.all <- K * J
+Y.DA <- Y   
+
+ 
+#### Create a missing list
+    if(n.miss>0)
+    {
+    miss.locator <- array(NA, c(n.miss, 2))
+    colnames(miss.locator) <- c("row", "column")
+    locations <- which(t(which.miss)==0)
+    miss.locator[ ,1] <- ceiling(locations/J)
+    miss.locator[ ,2] <- locations - (miss.locator[ ,1]-1) * J
+    }else
+    {}
+
 
-    
 #### Check on MALA argument
 if(length(MALA)!=1) stop("MALA is not length 1.", call.=FALSE)
 if(!is.logical(MALA)) stop("MALA is not logical.", call.=FALSE)  
@@ -30,18 +44,22 @@ if(!is.logical(MALA)) stop("MALA is not logical.", call.=FALSE)
 
 #### W matrix
     if(!is.matrix(W)) stop("W is not a matrix.", call.=FALSE)
-K <- nrow(W)
-    if(ceiling(N.all/K)!= floor(N.all/K)) stop("The number of data points divided by the number of rows in W is not a whole number.", call.=FALSE)
-J <- N.all / K
+    if(nrow(W)!= K) stop("The number of data points divided by the number of rows in W is not a whole number.", call.=FALSE)
 
 
-#### Check for errors on rho and fix.rho
-    if(!is.logical(fix.rho)) stop("fix.rho is not logical.", call.=FALSE)   
-    if(fix.rho & is.null(rho)) stop("rho is fixed but an initial value was not set.", call.=FALSE)   
-    if(fix.rho & !is.numeric(rho) ) stop("rho is fixed but is not numeric.", call.=FALSE)  
-    if(!fix.rho) rho <- runif(1)
+
+#### rho
+    if(is.null(rho))
+    {
+    rho <- runif(1)
+    fix.rho <- FALSE   
+    }else
+    {
+    fix.rho <- TRUE    
+    }
+    if(!is.numeric(rho) ) stop("rho is fixed but is not numeric.", call.=FALSE)  
     if(rho<0 ) stop("rho is outside the range [0, 1].", call.=FALSE)  
-    if(rho>1 ) stop("rho is outside the range [0, 1].", call.=FALSE)    
+    if(rho>1 ) stop("rho is outside the range [0, 1].", call.=FALSE)  
     
 
 #### Priors
@@ -74,41 +92,47 @@ common.burnin.nsample.thin.check(burnin, n.sample, thin)
 #############################
 #### Initial parameter values
 #############################
-mod.glm <- glm(Y~X.standardised-1, offset=offset, family="quasipoisson")
-beta.mean <- mod.glm$coefficients
-beta.sd <- sqrt(diag(summary(mod.glm)$cov.scaled))
-beta <- rnorm(n=p, mean=beta.mean, sd=beta.sd)
-    
+beta <- array(NA, c(p, J))
+    for(i in 1:J)
+    {
+    mod.glm <- glm(Y[ ,i]~X.standardised-1, offset=offset[ ,i], family="quasipoisson")
+    beta.mean <- mod.glm$coefficients
+    beta.sd <- sqrt(diag(summary(mod.glm)$cov.scaled))
+    beta[ ,i] <- rnorm(n=p, mean=beta.mean, sd=beta.sd)
+    }
+
 log.Y <- log(Y)
 log.Y[Y==0] <- -0.1  
-res.temp <- log.Y - X.standardised %*% beta.mean - offset
+res.temp <- log.Y - X.standardised %*% beta - offset
 res.sd <- sd(res.temp, na.rm=TRUE)/5
-phi <- rnorm(n=N.all, mean=0, sd=res.sd)
-phi.mat <- matrix(phi, nrow=K, byrow=TRUE)
-Sigma <- cov(phi.mat)
+phi.vec <- rnorm(n=N.all, mean=0, sd=res.sd)
+phi <- matrix(phi.vec, nrow=K, byrow=TRUE)
+Sigma <- cov(phi)
 Sigma.inv <- solve(Sigma)
+regression <- X.standardised %*% beta
+fitted <- exp(regression + phi + offset)
+    
 
-    
-    
 ###############################    
 #### Set up the MCMC quantities    
 ###############################
 #### Matrices to store samples    
 n.keep <- floor((n.sample - burnin)/thin)
-samples.beta <- array(NA, c(n.keep, p))
+samples.beta <- array(NA, c(n.keep, J*p))
 samples.phi <- array(NA, c(n.keep, N.all))
 samples.Sigma <- array(NA, c(n.keep, J, J))
     if(!fix.rho) samples.rho <- array(NA, c(n.keep, 1))
-samples.deviance <- array(NA, c(n.keep, 1))
 samples.like <- array(NA, c(n.keep, N.all))
 samples.fitted <- array(NA, c(n.keep, N.all))
     if(n.miss>0) samples.Y <- array(NA, c(n.keep, n.miss))
 
     
 #### Metropolis quantities
-accept.all <- rep(0,6)
+accept.all <- rep(0,4)
 accept <- accept.all
-proposal.sd.beta <- 0.01
+accept.all.beta <- rep(0,2*J)
+accept.beta <- accept.all.beta
+proposal.sd.beta <- rep(0.01, J)
 proposal.sd.phi <- 0.1
 proposal.sd.rho <- 0.02
 Sigma.post.df <- prior.Sigma.df + K  
@@ -148,14 +172,9 @@ islands.all <- rep(islands,J)
 n.islands <- max(W.islands$nc)
     if(rho==1) Sigma.post.df <- prior.Sigma.df + K - n.islands   
 
-        
-#### Make matrix versions of the variables
-offset.mat <- matrix(offset, nrow=K, ncol=J, byrow=TRUE) 
-regression.mat <- matrix(X.standardised %*% beta, nrow=K, ncol=J, byrow=TRUE)
-Y.mat <- matrix(Y, nrow=K, ncol=J, byrow=TRUE)
-Y.mat.miss <- matrix(Y.miss, nrow=K, ncol=J, byrow=TRUE)
-which.miss.mat <- matrix(which.miss, nrow=K, ncol=J, byrow=TRUE)
     
+#### Specify vector variants
+Y.vec <- as.numeric(t(Y))
 
 
 ###########################
@@ -176,59 +195,65 @@ which.miss.mat <- matrix(which.miss, nrow=K, ncol=J, byrow=TRUE)
 #### Create the MCMC samples      
     for(j in 1:n.sample)
     {
+    ####################################
+    ## Sample from Y - data augmentation
+    ####################################
+        if(n.miss>0)
+        {
+        Y.DA[miss.locator] <- rpois(n=n.miss, lambda=fitted[miss.locator])    
+        }else
+        {}
+
+        
+        
     ###################
     ## Sample from beta
     ###################
     offset.temp <- phi + offset
-        if(p>2)
+        for(r in 1:J)
         {
-        temp <- poissonbetaupdateMALA(X.standardised, N.all, p, beta, offset.temp, Y.miss, prior.mean.beta, prior.var.beta, which.miss, n.beta.block, proposal.sd.beta, list.block)
-        }else
-        {
-        temp <- poissonbetaupdateRW(X.standardised, N.all, p, beta, offset.temp, Y.miss, prior.mean.beta, prior.var.beta, which.miss, proposal.sd.beta)
+            if(p>2)
+            {
+            temp <- poissonbetaupdateMALA(X.standardised, K, p, beta[ ,r], offset.temp[ ,r], Y.DA[ ,r], prior.mean.beta, prior.var.beta, n.beta.block, proposal.sd.beta[r], list.block)
+            }else
+            {
+            temp <- poissonbetaupdateRW(X.standardised, K, p, beta[ ,r], offset.temp[ ,r], Y.DA[ ,r], prior.mean.beta, prior.var.beta, proposal.sd.beta[r])
+            }
+        beta[ ,r] <- temp[[1]]
+        accept.beta[r] <- accept.beta[r] + temp[[2]]
+        accept.beta[(r+J)] <- accept.beta[(r+J)] + n.beta.block  
         }
-    beta <- temp[[1]]
-    accept[1] <- accept[1] + temp[[2]]
-    accept[2] <- accept[2] + n.beta.block  
-    regression.mat <- matrix(X.standardised %*% beta, nrow=K, ncol=J, byrow=TRUE)          
+regression <- X.standardised %*% beta          
 
     
 
-    
-    
-    
     ##################
     ## Sample from phi
     ##################
     den.offset <- rho * W.triplet.sum + 1 - rho
-    phi.offset <- regression.mat + offset.mat
+    phi.offset <- regression + offset
         if(MALA)
         {
-        temp1 <- poissonmcarupdateMALA(W.triplet, W.begfin, K, J, phi.mat, Y.mat.miss,  phi.offset, den.offset, Sigma.inv, rho, proposal.sd.phi, which.miss.mat)      
+        temp1 <- poissonmcarupdateMALA(W.triplet, W.begfin, K, J, phi, Y.DA,  phi.offset, den.offset, Sigma.inv, rho, proposal.sd.phi)      
         }else
         {
-        temp1 <- poissonmcarupdateRW(W.triplet, W.begfin, K, J, phi.mat, Y.mat.miss,  phi.offset, den.offset, Sigma.inv, rho, proposal.sd.phi, which.miss.mat)      
+        temp1 <- poissonmcarupdateRW(W.triplet, W.begfin, K, J, phi, Y.DA,  phi.offset, den.offset, Sigma.inv, rho, proposal.sd.phi)      
         }
-        if(rho<1)
+
+    phi <- temp1[[1]]
+        for(r in 1:J)
         {
-        phi.mat <- temp1[[1]] - mean(temp1[[1]])
-        phi <- as.numeric(t(phi.mat))
-        }else
-        {
-        phi.mat <- temp1[[1]]
-        phi <- as.numeric(t(phi.mat))
-        phi[which(islands.all==1)] <- phi[which(islands.all==1)] - mean(phi[which(islands.all==1)]) 
-        phi.mat <- matrix(phi, nrow=K, byrow=TRUE)
+        phi[ ,r] <- phi[ ,r] - mean(phi[ ,r])    
         }
-    accept[3] <- accept[3] + temp1[[2]]
-    accept[4] <- accept[4] + K    
+    accept[1] <- accept[1] + temp1[[2]]
+    accept[2] <- accept[2] + K    
 
         
         
     ####################
     ## Sample from Sigma
     ####################
-    Sigma.post.scale <- t(phi.mat) %*% Q %*% phi.mat + prior.Sigma.scale
+    Sigma.post.scale <- t(phi) %*% Q %*% phi + prior.Sigma.scale
     Sigma <- riwish(Sigma.post.df, Sigma.post.scale)
     Sigma.inv <- solve(Sigma)
         
@@ -245,18 +270,18 @@ which.miss.mat <- matrix(which.miss, nrow=K, ncol=J, byrow=TRUE)
         det.Q.prop <-  sum(log((proposal.rho * Wstar.val + (1-proposal.rho))))    
             
         ## Compute the acceptance rate
-        logprob.current <- 0.5 * J * det.Q - 0.5 * sum(diag(t(phi.mat) %*% Q %*% phi.mat %*% Sigma.inv))
-        logprob.proposal <- 0.5 * J * det.Q.prop - 0.5 * sum(diag(t(phi.mat) %*% Q.prop %*% phi.mat %*% Sigma.inv))
+        logprob.current <- 0.5 * J * det.Q - 0.5 * sum(diag(t(phi) %*% Q %*% phi %*% Sigma.inv))
+        logprob.proposal <- 0.5 * J * det.Q.prop - 0.5 * sum(diag(t(phi) %*% Q.prop %*% phi %*% Sigma.inv))
         prob <- exp(logprob.proposal - logprob.current)
             if(prob > runif(1))
             {
             rho <- proposal.rho
             det.Q <- det.Q.prop
             Q <- Q.prop
-            accept[5] <- accept[5] + 1           
+            accept[3] <- accept[3] + 1           
             }else
             {}              
-        accept[6] <- accept[6] + 1       
+        accept[4] <- accept[4] + 1       
         }else
         {}
         
@@ -265,9 +290,8 @@ which.miss.mat <- matrix(which.miss, nrow=K, ncol=J, byrow=TRUE)
     #########################
     ## Calculate the deviance
     #########################
-    lp <- as.numeric(X.standardised %*% beta) + phi + offset
-    fitted <- exp(lp)
-    deviance.all <- dpois(x=as.numeric(Y), lambda=fitted, log=TRUE)
+    fitted <- exp(regression + phi + offset)
+    deviance.all <- dpois(x=as.numeric(t(Y)), lambda=as.numeric(t(fitted)), log=TRUE)
     like <- exp(deviance.all)
     deviance <- -2 * sum(deviance.all, na.rm=TRUE)  
         
@@ -278,14 +302,13 @@ which.miss.mat <- matrix(which.miss, nrow=K, ncol=J, byrow=TRUE)
         if(j > burnin & (j-burnin)%%thin==0)
         {
         ele <- (j - burnin) / thin
-        samples.beta[ele, ] <- beta
-        samples.phi[ele, ] <- phi
+        samples.beta[ele, ] <- as.numeric(beta)
+        samples.phi[ele, ] <- as.numeric(t(phi))
         samples.Sigma[ele, , ] <- Sigma
             if(!fix.rho) samples.rho[ele, ] <- rho
-        samples.deviance[ele, ] <- deviance
         samples.like[ele, ] <- like
-        samples.fitted[ele, ] <- fitted
-            if(n.miss>0) samples.Y[ele, ] <- rpois(n=n.miss, lambda=fitted[which.miss==0])
+        samples.fitted[ele, ] <- as.numeric(t(fitted))
+            if(n.miss>0) samples.Y[ele, ] <- Y.DA[miss.locator]
         }else
         {}
         
@@ -298,24 +321,29 @@ which.miss.mat <- matrix(which.miss, nrow=K, ncol=J, byrow=TRUE)
         if(ceiling(k)==floor(k))
         {
         #### Update the proposal sds
-            if(p>2)
+            for(r in 1:J)
             {
-            proposal.sd.beta <- common.accceptrates1(accept[1:2], proposal.sd.beta, 40, 50)
-            }else
-            {
-            proposal.sd.beta <- common.accceptrates1(accept[1:2], proposal.sd.beta, 30, 40)    
+                if(p>2)
+                {
+                proposal.sd.beta[r] <- common.accceptrates1(accept.beta[c(r, (r+J))], proposal.sd.beta[r], 40, 50)
+                }else
+                {
+                proposal.sd.beta[r] <- common.accceptrates1(accept.beta[c(r, (r+J))], proposal.sd.beta[r], 30, 40)    
+                }
             }
-        proposal.sd.phi <- common.accceptrates1(accept[3:4], proposal.sd.phi, 40, 50)
+            
+        proposal.sd.phi <- common.accceptrates1(accept[1:2], proposal.sd.phi, 40, 50)
             if(!fix.rho)
             {
-            proposal.sd.rho <- common.accceptrates2(accept[5:6], proposal.sd.rho, 40, 50, 0.5)
+            proposal.sd.rho <- common.accceptrates2(accept[3:4], proposal.sd.rho, 40, 50, 0.5)
             }
         accept.all <- accept.all + accept
-        accept <- c(0,0,0,0,0,0)
+        accept <- c(0,0,0,0)
+        accept.all.beta <- accept.all.beta + accept.beta
+        accept.beta <- rep(0,2*J)
         }else
         {}
-        
-        
+
         
     ################################       
     ## print progress to the console
@@ -336,16 +364,15 @@ which.miss.mat <- matrix(which.miss, nrow=K, ncol=J, byrow=TRUE)
     {}
 
 
-
 ###################################
 #### Summarise and save the results 
 ###################################
 #### Compute the acceptance rates
-accept.beta <- 100 * accept.all[1] / accept.all[2]
-accept.phi <- 100 * accept.all[3] / accept.all[4]
+accept.beta <- 100 * sum(accept.all.beta[1:J]) / sum(accept.all.beta[(J+1):(2*J)])
+accept.phi <- 100 * accept.all[1] / accept.all[2]
     if(!fix.rho)
     {
-    accept.rho <- 100 * accept.all[5] / accept.all[6]
+    accept.rho <- 100 * accept.all[3] / accept.all[4]
     }else
     {
     accept.rho <- NA    
@@ -355,44 +382,45 @@ accept.final <- c(accept.beta, accept.phi, accept.rho, accept.Sigma)
 names(accept.final) <- c("beta", "phi", "rho", "Sigma")
     
 
-#### Deviance information criterion (DIC)
-median.beta <- apply(samples.beta, 2, median)
-median.phi <- apply(samples.phi, 2, median)
-fitted.median <- exp(X.standardised %*% median.beta + median.phi + offset)
-deviance.fitted <- -2 * sum(dpois(x=Y, lambda=fitted.median, log=TRUE), na.rm=TRUE)
-p.d <- median(samples.deviance) - deviance.fitted
-DIC <- 2 * median(samples.deviance) - deviance.fitted     
+#### Compute the fitted deviance
+mean.beta <- matrix(apply(samples.beta, 2, mean), nrow=p, ncol=J, byrow=F)
+mean.phi <- matrix(apply(samples.phi, 2, mean), nrow=K, ncol=J, byrow=T)
+fitted.mean <- exp(X.standardised %*% mean.beta + mean.phi + offset)
+deviance.fitted <- -2 * sum(dpois(x=as.numeric(t(Y)), lambda=as.numeric(t(fitted.mean)), log=TRUE), na.rm=TRUE)
+
     
-    
-#### Watanabe-Akaike Information Criterion (WAIC)
-LPPD <- sum(log(apply(samples.like,2,mean)), na.rm=TRUE)
-p.w <- sum(apply(log(samples.like),2,var), na.rm=TRUE)
-WAIC <- -2 * (LPPD - p.w)
-    
-#### Compute the Conditional Predictive Ordinate
-CPO <- rep(NA, N.all)
-    for(j in 1:N.all)
-    {
-    CPO[j] <- 1/median((1 / dpois(x=Y[j], lambda=samples.fitted[ ,j])))    
-    }
-LMPL <- sum(log(CPO), na.rm=TRUE)  
-    
-    
-#### Compute the % deviance explained
-fit.null <- glm(Y~1, offset=offset, family="poisson")$fitted.values
-deviance.null <- -2 * sum(dpois(x=Y[which(!is.na(Y))], lambda=fit.null, log=TRUE), na.rm=TRUE)
-percent_dev_explained <- 100 * (deviance.null - deviance.fitted) / deviance.null
+#### Model fit criteria
+modelfit <- common.modelfit(samples.like, deviance.fitted)
 
 
 #### transform the parameters back to the origianl covariate scale.
-samples.beta.orig <- common.betatransform(samples.beta, X.indicator, X.mean, X.sd, p, FALSE)
+samples.beta.orig <- samples.beta
+    for(r in 1:J)
+    {
+    samples.beta.orig[ ,((r-1)*p+1):(r*p)] <- common.betatransform(samples.beta[ ,((r-1)*p+1):(r*p) ], X.indicator, X.mean, X.sd, p, FALSE)
+    }
 
 
 #### Create a summary object
 samples.beta.orig <- mcmc(samples.beta.orig)
 summary.beta <- t(apply(samples.beta.orig, 2, quantile, c(0.5, 0.025, 0.975))) 
-summary.beta <- cbind(summary.beta, rep(n.keep, p), rep(accept.beta,p), effectiveSize(samples.beta.orig), geweke.diag(samples.beta.orig)$z)
-rownames(summary.beta) <- colnames(X)
+summary.beta <- cbind(summary.beta, rep(n.keep, p), rep(accept.beta,J*p), effectiveSize(samples.beta.orig), geweke.diag(samples.beta.orig)$z)
+col.name <- rep(NA, p*(J-1))
+
+if(is.null(colnames(Y)))
+{
+    for(r in 1:J)
+    {
+        col.name[((r-1)*p+1):(r*p)] <- paste("Category ", r+1,  " - ", colnames(X), sep="")   
+    }
+}else
+{
+    for(r in 1:J)
+    {
+        col.name[((r-1)*p+1):(r*p)] <- paste(colnames(Y)[r],  " - ", colnames(X), sep="")   
+    }
+}
+rownames(summary.beta) <- col.name
 colnames(summary.beta) <- c("Median", "2.5%", "97.5%", "n.sample", "% accept", "n.effective", "Geweke.diag")
     
 summary.hyper <- array(NA, c((J+1) ,7))
@@ -418,30 +446,27 @@ summary.hyper[1:J, 6] <- diag(apply(samples.Sigma, c(2,3), effectiveSize))
     }
     
 summary.results <- rbind(summary.beta, summary.hyper)
-rownames(summary.results)[(p+1): nrow(summary.results)] <- c(paste(rep("Sigma",J), 1:J, 1:J, sep=""), "rho")
+rownames(summary.results)[((J*p)+1): nrow(summary.results)] <- c(paste(rep("Sigma",J), 1:J, 1:J, sep=""), "rho")
 summary.results[ , 1:3] <- round(summary.results[ , 1:3], 4)
 summary.results[ , 4:7] <- round(summary.results[ , 4:7], 1)
     
     
 #### Create the fitted values and residuals
-fitted.values <- apply(samples.fitted, 2, median)
-response.residuals <- as.numeric(Y) - fitted.values
-pearson.residuals <- response.residuals /sqrt(fitted.values)
-deviance.residuals <- sign(response.residuals) * sqrt(2 * (Y * log(Y/fitted.values) + fitted.values - Y))
-residuals <- data.frame(response=response.residuals, pearson=pearson.residuals, deviance=deviance.residuals)
-    
+fitted.values <- matrix(apply(samples.fitted, 2, mean), nrow=K, ncol=J, byrow=T)
+response.residuals <- Y - fitted.values
+pearson.residuals <- response.residuals / sqrt(fitted.values)
+residuals <- list(response=response.residuals, pearson=pearson.residuals)
+
+
     
 #### Compile and return the results
-loglike <- (-0.5 * deviance.fitted)
-modelfit <- c(DIC, p.d, WAIC, p.w, LMPL, loglike, percent_dev_explained)
-names(modelfit) <- c("DIC", "p.d", "WAIC", "p.w", "LMPL", "loglikelihood", "Percentage deviance explained")
 model.string <- c("Likelihood model - Poisson (log link function)", "\nRandom effects model - Leroux MCAR\n")
     if(fix.rho) samples.rho=NA
     if(n.miss==0) samples.Y = NA
 samples <- list(beta=samples.beta.orig, phi=mcmc(samples.phi), Sigma=samples.Sigma, rho=mcmc(samples.rho), fitted=mcmc(samples.fitted), Y=mcmc(samples.Y))
 results <- list(summary.results=summary.results, samples=samples, fitted.values=fitted.values, residuals=residuals, modelfit=modelfit, accept=accept.final, localised.structure=NULL,  formula=formula, model=model.string, X=X)
 class(results) <- "CARBayes"
-    
+
 
 #### Finish by stating the time taken    
     if(verbose)
