@@ -18,9 +18,9 @@ X.mean <- frame.results$X.mean
 X.indicator <- frame.results$X.indicator 
 offset <- frame.results$offset
 Y <- frame.results$Y
-Y.miss <- frame.results$Y.miss
 which.miss <- frame.results$which.miss
 n.miss <- frame.results$n.miss  
+Y.DA <- Y   
 
 
 #### Check on MALA argument
@@ -71,7 +71,7 @@ phi <- rnorm(n=K, mean=rep(0,K), sd=res.sd)
 tau2 <- var(phi) / 10
 theta <- rnorm(n=K, mean=rep(0,K), sd=res.sd)
 sigma2 <- var(theta) / 10
-
+fitted <- exp(as.numeric(X.standardised %*% beta) + phi + theta + offset)
 
 
 ###############################    
@@ -83,7 +83,6 @@ samples.beta <- array(NA, c(n.keep, p))
 samples.re <- array(NA, c(n.keep, K))
 samples.tau2 <- array(NA, c(n.keep, 1))
 samples.sigma2 <- array(NA, c(n.keep, 1))
-samples.deviance <- array(NA, c(n.keep, 1))
 samples.like <- array(NA, c(n.keep, K))
 samples.fitted <- array(NA, c(n.keep, K))
     if(n.miss>0) samples.Y <- array(NA, c(n.keep, n.miss))
@@ -140,16 +139,27 @@ tau2.posterior.shape <- prior.tau2[1] + 0.5 * (K-n.islands)
 #### Create the MCMC samples     
     for(j in 1:n.sample)
     {
+    ####################################
+    ## Sample from Y - data augmentation
+    ####################################
+        if(n.miss>0)
+        {
+        Y.DA[which.miss==0] <- rpois(n=n.miss, lambda=fitted[which.miss==0])    
+        }else
+        {}
+        
+        
+        
     ####################
     ## Sample from beta
     ####################
     offset.temp <- phi + offset + theta
         if(p>2)
         {
-        temp <- poissonbetaupdateMALA(X.standardised, K, p, beta, offset.temp, Y.miss, prior.mean.beta, prior.var.beta, which.miss, n.beta.block, proposal.sd.beta, list.block)
+        temp <- poissonbetaupdateMALA(X.standardised, K, p, beta, offset.temp, Y.DA, prior.mean.beta, prior.var.beta, n.beta.block, proposal.sd.beta, list.block)
         }else
         {
-        temp <- poissonbetaupdateRW(X.standardised, K, p, beta, offset.temp, Y.miss, prior.mean.beta, prior.var.beta, which.miss, proposal.sd.beta)
+        temp <- poissonbetaupdateRW(X.standardised, K, p, beta, offset.temp, Y.DA, prior.mean.beta, prior.var.beta, proposal.sd.beta)
         }
     beta <- temp[[1]]
     accept[1] <- accept[1] + temp[[2]]
@@ -163,15 +173,15 @@ tau2.posterior.shape <- prior.tau2[1] + 0.5 * (K-n.islands)
     beta.offset <- X.standardised %*% beta + theta + offset
         if(MALA)
         {
-        temp1 <- poissoncarupdateMALA(Wtriplet=W.triplet, Wbegfin=W.begfin, W.triplet.sum, nsites=K, phi=phi, tau2=tau2, y=Y.miss, phi_tune=proposal.sd.phi, rho=1, offset=beta.offset, which.miss)
+        temp1 <- poissoncarupdateMALA(Wtriplet=W.triplet, Wbegfin=W.begfin, W.triplet.sum, nsites=K, phi=phi, tau2=tau2, y=Y.DA, phi_tune=proposal.sd.phi, rho=1, offset=beta.offset)
         }else
         {
-        temp1 <- poissoncarupdateRW(Wtriplet=W.triplet, Wbegfin=W.begfin, W.triplet.sum, nsites=K, phi=phi, tau2=tau2, y=Y.miss, phi_tune=proposal.sd.phi, rho=1, offset=beta.offset, which.miss)
+        temp1 <- poissoncarupdateRW(Wtriplet=W.triplet, Wbegfin=W.begfin, W.triplet.sum, nsites=K, phi=phi, tau2=tau2, y=Y.DA, phi_tune=proposal.sd.phi, rho=1, offset=beta.offset)
         }
     phi <- temp1[[1]]
     phi[which(islands==1)] <- phi[which(islands==1)] - mean(phi[which(islands==1)])
     accept[3] <- accept[3] + temp1[[2]]
-    accept[4] <- accept[4] + K - n.miss    
+    accept[4] <- accept[4] + K    
 
     
        
@@ -181,15 +191,15 @@ tau2.posterior.shape <- prior.tau2[1] + 0.5 * (K-n.islands)
     beta.offset <- as.numeric(X.standardised %*% beta) + phi + offset
         if(MALA)
         {
-        temp2 <- poissonindepupdateMALA(nsites=K, theta=theta, sigma2=sigma2, y=Y.miss, theta_tune=proposal.sd.theta, offset=beta.offset, which.miss) 
+        temp2 <- poissonindepupdateMALA(nsites=K, theta=theta, sigma2=sigma2, y=Y.DA, theta_tune=proposal.sd.theta, offset=beta.offset) 
         }else
         {
-        temp2 <- poissonindepupdateRW(nsites=K, theta=theta, sigma2=sigma2, y=Y.miss, theta_tune=proposal.sd.theta, offset=beta.offset, which.miss) 
+        temp2 <- poissonindepupdateRW(nsites=K, theta=theta, sigma2=sigma2, y=Y.DA, theta_tune=proposal.sd.theta, offset=beta.offset) 
         }
     theta <- temp2[[1]]
     theta <- theta - mean(theta)    
     accept[5] <- accept[5] + temp2[[2]]
-    accept[6] <- accept[6] + K - n.miss
+    accept[6] <- accept[6] + K
 
   
 
@@ -231,10 +241,9 @@ tau2.posterior.shape <- prior.tau2[1] + 0.5 * (K-n.islands)
         samples.re[ele, ] <- phi + theta
         samples.tau2[ele, ] <- tau2
         samples.sigma2[ele, ] <- sigma2
-        samples.deviance[ele, ] <- deviance
         samples.like[ele, ] <- like
         samples.fitted[ele, ] <- fitted
-            if(n.miss>0) samples.Y[ele, ] <- rpois(n=n.miss, lambda=fitted[which.miss==0])
+            if(n.miss>0) samples.Y[ele, ] <- Y.DA[which.miss==0]
         }else
         {
         }
@@ -297,34 +306,15 @@ accept.final <- c(accept.beta, accept.phi, accept.theta, accept.tau2, accept.sig
 names(accept.final) <- c("beta", "phi", "theta", "tau2", "sigma2")
 
      
-#### Deviance information criterion (DIC)
-median.beta <- apply(samples.beta, 2, median)
-median.re <- apply(samples.re, 2, median)
-fitted.median <- exp(X.standardised %*% median.beta + median.re  + offset)
-deviance.fitted <- -2 * sum(dpois(x=Y, lambda=fitted.median, log=TRUE), na.rm=TRUE)
-p.d <- median(samples.deviance) - deviance.fitted
-DIC <- 2 * median(samples.deviance) - deviance.fitted     
-     
-     
-#### Watanabe-Akaike Information Criterion (WAIC)
-LPPD <- sum(log(apply(samples.like,2,mean)), na.rm=TRUE)
-p.w <- sum(apply(log(samples.like),2,var), na.rm=TRUE)
-WAIC <- -2 * (LPPD - p.w)
-
-
-#### Compute the Conditional Predictive Ordinate
-CPO <- rep(NA, K)
-    for(j in 1:K)
-    {
-    CPO[j] <- 1/median((1 / dpois(x=Y[j], lambda=samples.fitted[ ,j])))    
-    }
-LMPL <- sum(log(CPO), na.rm=TRUE)  
+#### Compute the fitted deviance
+mean.beta <- apply(samples.beta, 2, mean)
+mean.re <- apply(samples.re, 2, mean)
+fitted.mean <- exp(X.standardised %*% mean.beta + mean.re  + offset)
+deviance.fitted <- -2 * sum(dpois(x=Y, lambda=fitted.mean, log=TRUE), na.rm=TRUE)
 
      
-#### Compute the % deviance explained
-fit.null <- glm(Y~1, offset=offset, family="poisson")$fitted.values
-deviance.null <- -2 * sum(dpois(x=Y[which(!is.na(Y))], lambda=fit.null, log=TRUE), na.rm=TRUE)
-percent_dev_explained <- 100 * (deviance.null - deviance.fitted) / deviance.null
+#### Model fit criteria
+modelfit <- common.modelfit(samples.like, deviance.fitted)
 
 
 #### transform the parameters back to the origianl covariate scale.
@@ -352,17 +342,13 @@ summary.results[ , 4:7] <- round(summary.results[ , 4:7], 1)
 
 
 #### Create the fitted values and residuals
-fitted.values <- apply(samples.fitted, 2, median)
+fitted.values <- apply(samples.fitted, 2, mean)
 response.residuals <- as.numeric(Y) - fitted.values
 pearson.residuals <- response.residuals /sqrt(fitted.values)
-deviance.residuals <- sign(response.residuals) * sqrt(2 * (Y * log(Y/fitted.values) + fitted.values - Y))
-residuals <- data.frame(response=response.residuals, pearson=pearson.residuals, deviance=deviance.residuals)
+residuals <- data.frame(response=response.residuals, pearson=pearson.residuals)
 
 
 #### Compile and return the results
-loglike <- (-0.5 * deviance.fitted)
-modelfit <- c(DIC, p.d, WAIC, p.w, LMPL, loglike, percent_dev_explained)
-names(modelfit) <- c("DIC", "p.d", "WAIC", "p.w", "LMPL", "loglikelihood", "Percentage deviance explained")
 model.string <- c("Likelihood model - Poisson (log link function)", "\nRandom effects model - BYM CAR\n")
 if(n.miss==0) samples.Y = NA
 
